#include "ElementUpdate_2d.h"

double 
ElementUpdate_2d::calcElementUpdate(
                  const FixVec<WorldVector<double> *, VERTEX> &vert,
		  FixVec<double, VERTEX> &uhVal)
{
  WorldVector<double> xhminusYh = *(vert[2]) - *(vert[0]);
  WorldVector<double> zhminusYh = *(vert[1]) - *(vert[0]);
  WorldVector<double> xhminusZh = *(vert[2]) - *(vert[1]);
  
  double norm_zhminusYh = sqrt(zhminusYh * zhminusYh);
  double norm_xhminusYh = sqrt(xhminusYh * xhminusYh);
  double norm_xhminusZh = sqrt(xhminusZh * xhminusZh);
  
  double delta = (uhVal[1]-uhVal[0]) / norm_zhminusYh;
  
  double c_alpha = 0;
  double c_beta = 0;
  double sP = xhminusYh * zhminusYh;
  c_alpha = sP / (norm_xhminusYh * norm_zhminusYh);
  sP = xhminusZh * zhminusYh;
  c_beta = -sP / (norm_xhminusZh * norm_zhminusYh);
  
  double update = 0;
  if (c_alpha <= delta) {
    update = uhVal[0] + norm_xhminusYh;
    //save barycentric coordinates for calculation of the velocity
    if(velExt != NULL)
      {
	velExt->setBarycentricCoords_2D(1,0,0);
      }
  } else if (delta <= -c_beta) {
    update = uhVal[1] + norm_xhminusZh;
    //save barycentric coordinates for calculation of the velocity
    if(velExt != NULL)
      {
	velExt->setBarycentricCoords_2D(0,1,0);
      }
  } else {
    update = uhVal[0] + 
      (c_alpha*delta + sqrt((1-c_alpha*c_alpha)*(1-delta*delta))) *
      norm_xhminusYh;
    //calculate and save barycentric coordinates for calculation of the velocity
    if(velExt != NULL)
      {
	velExt->calcBarycentricCoords_2D(delta, c_alpha, norm_zhminusYh, norm_xhminusYh);
      }
  }
  
  return update;
}
